@@ -21,13 +21,21 @@
 #include "HashTable.h"
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
+#ifndef _MSC_VER
 #include <libgen.h>
 #include <sys/param.h>
+#include <unistd.h>
+#else
+#define MAXPATHLEN _MAX_PATH
+#include <direct.h>
+char* dirname(const char*);
+void realpath(const char*,char*);
+#endif
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <errno.h>
-#include <unistd.h>
-#include <stdio.h>
+
 
 #ifdef SP_NAMESPACE
 namespace SP_NAMESPACE {
@@ -106,7 +114,7 @@ XmlOutputEventHandler::XmlOutputEventHandler(const Options &options,
   // Open file for writing external entity declarations if we are preserving
   // any entities
   if (! options_.expExt) {
-    ParserApp::AppChar filePath[strlen(outputDir_) + 21];
+    ParserApp::AppChar* filePath=new ParserApp::AppChar[strlen(outputDir_) + 21];
     strcpy (filePath, outputDir_);
     strcat (filePath, "/");
     strcat (filePath, EXT_ENT_FILE);
@@ -135,12 +143,13 @@ XmlOutputEventHandler::XmlOutputEventHandler(const Options &options,
     /* Write some header information to this file. */
     *extEnts_ << "<!-- This file automatically generated by osx -->\n";
     extEnts_->flush();
+	delete [] filePath;
   }
 
   // Open file for writing external entity declarations if we are preserving
   // any entities
   if (! options_.expInt) {
-    char filePath[strlen(outputDir_) + 21];
+    char* filePath=new char[strlen(outputDir_) + 21];
     strcpy (filePath, outputDir_);
     strcat (filePath, "/");
     strcat (filePath, INT_ENT_FILE);
@@ -169,6 +178,7 @@ XmlOutputEventHandler::XmlOutputEventHandler(const Options &options,
     /* Write some header information to this file. */
     *intEnts_ << "<!-- This file automatically generated by osx -->\n";
     intEnts_->flush();
+	delete [] filePath;
   }
 
     /* If we are expanding internal entities, we are definitely not
@@ -1153,7 +1163,7 @@ void XmlOutputEventHandler::inputOpened(InputSource *in)
 	     becomes ./usr/local/lib/ents/foo.xml; possible 2-digit
              suffix (for uniqueness) */
 
-	  char filePath[strlen(outputDir_) + 9 +
+	  char* filePath=new char[strlen(outputDir_) + 9 +
 		       outputCodingSystem->convertOut
 			(*systemIdPointer).size()];
 
@@ -1189,12 +1199,22 @@ void XmlOutputEventHandler::inputOpened(InputSource *in)
           // Check to make sure we haven't passed outside of the
           // output directory
 	  char *dirs = strdup (filePath);
+#ifdef MAXPATHLEN
           char realDirs[MAXPATHLEN];
           char realOutputDir[MAXPATHLEN];
+#else
+          char *realDirs;
+          char *realOutputDir;
+#endif
           char *outputDir = strdup(outputDir_);
 
+#ifdef MAXPATHLEN
           realpath((const char *)dirname(dirs), realDirs);
           realpath((const char *)dirname(outputDir), realOutputDir);
+#else
+          realDirs = realpath((const char *)dirname(dirs), NULL);
+          realOutputDir = realpath((const char *)dirname(outputDir), NULL);
+#endif
 
           if (strncmp(realDirs, realOutputDir, strlen (realOutputDir)) != 0) {
             app_->message(XmlOutputMessages::pathOutsideOutputDirectory,
@@ -1204,6 +1224,11 @@ void XmlOutputEventHandler::inputOpened(InputSource *in)
             }
           }
 
+#ifndef MAXPATHLEN
+	  free(realDirs);
+	  free(realOutputDir);
+#endif
+
 	  // Make the necessary directories
 	  maybeCreateDirectories(dirname(dirs));
 
@@ -1258,6 +1283,7 @@ void XmlOutputEventHandler::inputOpened(InputSource *in)
 	  // Create output stream to file and set os_ to it.
 	  os_ = (OutputCharStream *)
 	    new EncodeOutputCharStream(file, outputCodingSystem);
+	  delete [] filePath;
 	} // end if firstSeen
 	else {
 	  // push null os onto file output stack, set os_ to it
@@ -1447,6 +1473,10 @@ int XmlOutputEventHandler::maybeCreateDirectories (char *path)
   int ret;
 
   char *pp = strrchr(path, '/');
+#ifdef SP_MSDOS_FILENAMES
+  	char *ppp = strrchr(path,'\\');
+	if (ppp && (ppp > pp || pp==NULL)) pp=ppp;
+#endif
 
   if ((pp) && (pp != path)) {
     *pp = '\0';
@@ -1454,6 +1484,7 @@ int XmlOutputEventHandler::maybeCreateDirectories (char *path)
     *pp = '/';
   }
 
+#ifndef _MSC_VER
   struct stat statbuf;
   pp = strrchr(path, '/');
 
@@ -1467,6 +1498,9 @@ int XmlOutputEventHandler::maybeCreateDirectories (char *path)
   }
 
   ret = mkdir(path, statbuf.st_mode);
+#else
+  ret=_mkdir(path);
+#endif
   if (ret == -1) {
     if (errno == EEXIST) {
       ret = 0;
@@ -1505,7 +1539,7 @@ Boolean XmlOutputEventHandler::checkFirstSeen(const StringC &name)
  */
 void XmlOutputEventHandler::uniqueFilename(char *filePath) {
 
-  char baseFilePath[strlen(filePath)];
+  char* baseFilePath=new char[strlen(filePath)];
   strcpy (baseFilePath, filePath);
 
   struct stat statbuf;
@@ -1526,6 +1560,7 @@ void XmlOutputEventHandler::uniqueFilename(char *filePath) {
                   (app_->codingSystem()->convertIn(baseFilePath)));
     exit(1);
   }
+  delete [] baseFilePath;
 }
 
 char XmlOutputEventHandler::getQuoteMark(const StringC *contents)
@@ -1628,3 +1663,27 @@ void NullOutputByteStream::flushBuf(char)
 #ifdef SP_NAMESPACE
 }
 #endif
+
+#ifdef _MSC_VER
+char* dirname(const char* path)
+{
+	static char buffer[MAXPATHLEN];
+	char* p=strrchr((char*)path,'/');
+#ifdef SP_MSDOS_FILENAMES
+	char *ppp = strrchr((char*)path,'\\');
+	if (ppp && (ppp > p || p==NULL)) p=ppp;
+#endif
+ 
+	if (p==NULL) return NULL;
+	strncpy(buffer,path,p-path);
+	buffer[p-path]=0;
+	return buffer;
+}
+
+void realpath(const char* in,char* out)
+{
+	_fullpath(out,in,MAXPATHLEN);
+	return;
+}
+#endif
+
